#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>
#include <stdio.h>
#include "gmp.h"

int main(int argc, char *argv[]) {
  Gmp *ge1, *ge2;
  const char *error;
  int pass=0;
  GmpResult message;
  int i, j;
  int which_ge;
  int c;

  int size=19;
  int handicap=0;
  char white[100]="/u/martinea/gnugo/gnugo";
  char black[100]="/u/martinea/gnugo/gnugo";
  char white_output[100]="";
  char black_output[100]="";

  int gnugopipeI1[2];
  int gnugopipeO1[2];
  int gnugopipeI2[2];
  int gnugopipeO2[2];

/*c=getopt(argc, argv, "b:h:o:s:w:O:");
fprintf(stderr,"c=%c %d\n",c,c);*/

  while ((c=getopt(argc, argv, "b:h:o:s:w:O:")) != EOF) {
    switch (c) {
    case 'b' : strcpy(black, optarg);
               break;
    case 'h' : handicap = atoi(optarg);
               break;
    case 'o' : strcpy(white_output, optarg);
               break;
    case 's' : size = atoi(optarg);
               break;
    case 'w' : strcpy(white, optarg);
               break;
    case 'O' : strcpy(black_output, optarg);
               break;
    }
  }


  pipe(gnugopipeI1);
  pipe(gnugopipeO1);
  pipe(gnugopipeI2);
  pipe(gnugopipeO2);

  switch (fork()) {
  case -1:
    exit(-1);
  case 0:
    break;
  default:
    close(0);
    close(1);
    dup2(gnugopipeI1[0],0);
    dup2(gnugopipeO1[1],1);
    close(gnugopipeI1[0]);
    close(gnugopipeI1[1]);
    close(gnugopipeO1[0]);
    close(gnugopipeO1[1]);
    close(gnugopipeI2[0]);
    close(gnugopipeI2[1]);
    close(gnugopipeO2[0]);
    close(gnugopipeO2[1]);
    if(strlen(white_output)>0)
      execl(white,"gnugo","--mode", "gmp", "--quiet", "-o", white_output, NULL);
    else
      execl(white,"gnugo","--mode", "gmp", "--quiet", NULL);
    exit(0);
  }
  switch (fork()) {
  case -1:
    exit(-1);
  case 0:
    break;
  default:
    close(0);
    close(1);
    dup2(gnugopipeI2[0],0);
    dup2(gnugopipeO2[1],1);
    close(gnugopipeI1[0]);
    close(gnugopipeI1[1]);
    close(gnugopipeO1[0]);
    close(gnugopipeO1[1]);
    close(gnugopipeI2[0]);
    close(gnugopipeI2[1]);
    close(gnugopipeO2[0]);
    close(gnugopipeO2[1]);
    if(strlen(black_output)>0)
      execl(black,"gnugo","--mode", "gmp", "--quiet", "-o", black_output, NULL);
    else
      execl(black,"gnugo","--mode", "gmp", "--quiet", NULL);
    exit(0);
  }
  close(gnugopipeI1[0]);
  close(gnugopipeO1[1]);
  close(gnugopipeI2[0]);
  close(gnugopipeO2[1]);

fprintf(stderr, "black       : %s\n",black);
fprintf(stderr, "white       : %s\n",white);
fprintf(stderr, "black_output: %s\n",black_output);
fprintf(stderr, "white_output: %s\n",white_output);
fprintf(stderr, "size        : %d\n",size);
fprintf(stderr, "handicap    : %d\n",handicap);

  ge1 = gmp_create(gnugopipeO1[0], gnugopipeI1[1]);
  ge2 = gmp_create(gnugopipeO2[0], gnugopipeI2[1]);

  gmp_startGame(ge1,size,handicap,5.5,0,0); /* white */
  gmp_startGame(ge2,size,handicap,5.5,0,1); /* black */

  do {
    message = gmp_check(ge1,1,NULL,NULL,&error);
  } while (message != gmp_newGame);
/*  } while (message == gmp_nothing || message == gmp_reset); */

  do {
    message = gmp_check(ge2,1,NULL,NULL,&error);
  } while (message != gmp_newGame);
/*  } while (message == gmp_nothing || message == gmp_reset); */

  while(pass < 2) {
    message = gmp_check2(ge1, ge2, &which_ge, 1, &j, &i, &error);
    if(which_ge == 1) {
      if(message == gmp_pass) {
        pass++;
        gmp_sendPass(ge2);
      }
      else if(message == gmp_move) {
        pass=0;
        gmp_sendMove(ge2, j, i);
      }
      else if(message == gmp_newGame) {
        fprintf(stderr, "gmp_newGame from 1\n");
      }
      else if(message != gmp_nothing) {
        fprintf(stderr, "message from 1: %d\n", message);
	sleep(5);
	exit(0);
      }
    }
    else if(which_ge == 2) {
      if(message == gmp_pass) {
        pass++;
        gmp_sendPass(ge1);
      }
      else if(message == gmp_move) {
        pass=0;
        gmp_sendMove(ge1, j, i);
      }
      else if(message == gmp_newGame) {
        fprintf(stderr, "gmp_newGame from 2\n");
      }
      else if(message != gmp_nothing) {
        fprintf(stderr, "message from 2: %d\n", message);
	sleep(5);
	exit(0);
      }
    }
  }
  sleep(5);
  return 0;
}
